/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.util;

import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.interfaces.ICalcElement;
import isac.util.interfaces.ICalcIterator;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

/**
 * This class is responsible for the representation of a particular
 * <code>CalcElement</code> in a <code>CalcModelHierarchy</code>
 * 
 * @author Mario Hochreiter
 * @version 1.0
 * @version 1.1
 * @author Alois Kirchsteiger
 */
public class CalcModelNode implements Serializable {
  static Logger logger = Logger.getLogger(CalcModelNode.class.getName());

  protected ICalcIterator it_;
  protected Vector children_;
  protected CalcModelNode parent_;

  public CalcModelNode(ICalcIterator it) {
    it_ = it;
    children_ = new Vector();
    parent_ = null;
  }

  /**
   * @param child
   * @param index
   * @return true is successful, false if node could not be added at the specuified position
   */
  public boolean add(int index, CalcModelNode child) {
    try {
      children_.add(index, child);
      child.parent_ = this;
      return true;
    } catch (ArrayIndexOutOfBoundsException e) {
      return false;
    }
  }
  
  /**
   * inserts the child on the right position, which is determined by the position of the iterator
   * @param child
   * @return true if successful
   */
  public boolean add(CalcModelNode child) {
    Iterator it = children_.iterator();
    boolean successorFound = false;
    int index = 0;
    while (!successorFound && it.hasNext()) {
      CalcModelNode currentNode = (CalcModelNode)it.next();
      if (currentNode.getValue() == null) {
        successorFound = true;
      } else if (CalcModelHierarchy.compareIterators(currentNode.getValue(), child.getValue()) < 0) {
        index++;
      } else if (CalcModelHierarchy.compareIterators(currentNode.getValue(), child.getValue()) == 0) {
        return false; //node with the value already exists
      } else { // currentNode has a higher position than child
        successorFound = true;
      }
    }
    children_.add(index, child);
    child.parent_ = this;
    return true;
  }
  
  /**
   * @return the path to this node, where the root is at index 0 and the 
   * parent node is at index size-1
   */
  public Vector getPath() {
    Vector result = new Vector();
    if (parent_ != null) {
      result = parent_.getPath();
    }
    result.add(this);
    return result;
  }
  
  /**
   * @return the path to the parent node, where the root is at index 0 and the 
   * parent node is at index size-1
   */
  public Vector getParentPath() {
    Vector result = new Vector();
    if (parent_ != null) {
      result = parent_.getPath();
    }
    return result;
  }
  
  /**
   * @return the children of this node
   */
  public Vector getChildren() {
    return children_;
  }
  
  /**
   * @param index
   * @return the child at the specified index
   */
  public CalcModelNode getChild(int index) {
    return (CalcModelNode)children_.get(index);  
  }
  
  /**
   * removes the specified child
   */
  public void remove(CalcModelNode child) {
    child.parent_ = null;
    children_.remove(child);
  }
  
  /**
   * @return the parent <code>CalcModelNode</code> of this node
   */
  public CalcModelNode getParent() {
    return parent_;
  }
  
  /**
   * 
   * @return index of this node as seen from its parent
   */
  public int getIndex() {
    return parent_.getChildIndex(this);
  }
  
  /**
   * return the value of the node, which is a <code>ICalcIterator</code>
   */
  public ICalcIterator getValue() {
    return it_;
  }
 
  /** @return Visual representation of a <code>CalcElement</code> */
  public String toString() {
    if (it_ != null) {
      try {
        ICalcElement element = it_.getFormula();
        if (element instanceof CalcFormula) {
          return ((CalcFormula)element).toString();
        } else if (element instanceof CalcHead) {
          return ((CalcHead)element).getHead();
        } else {
          return null;
        }
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
    return "    ";
  }
  
  /**
   * compares a CalcModelNode to a object
   * if this object is also a CalcModelNode an its CalcElement is the
   * same, this method will return true
   */
  public boolean equals(Object o) {
    if (o instanceof CalcModelNode) {
      CalcModelNode node = (CalcModelNode) o;
      if (node.it_ != null && it_ != null) {
        return (CalcModelHierarchy.compareIterators(node.it_, it_) == 0);
      } else {
        return (it_ == null && node.it_ == null); //this.ce equals node.ce which is also null
      }
    }
    return false;
  }

  /**
   * @return
   */
  public int getChildCount() {
    return children_.size();
  }
  
  /**
   * @param child
   * @return index of the child
   */
  public int getChildIndex(CalcModelNode child) {
    return children_.indexOf(child);
  }
}
